import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

public class FastConvexHull
{
	public int execute(ArrayList<Vertex> vertices, HashMap<Vertex, Point2D> map) 
	{
		ArrayList<Vertex> xSorted = (ArrayList<Vertex>) vertices.clone();
		Collections.sort(xSorted, new XCompare(map));
		
		int n = xSorted.size();
		
		Vertex[] lUpper = new Vertex[n];
		
		lUpper[0] = xSorted.get(0);
		lUpper[1] = xSorted.get(1);
		
		int lUpperSize = 2;
		
		for (int i = 2; i < n; i++)
		{
			lUpper[lUpperSize] = xSorted.get(i);
			lUpperSize++;
			
			while (lUpperSize > 2 && !rightTurn(map.get(lUpper[lUpperSize - 3]), map.get(lUpper[lUpperSize - 2]), map.get(lUpper[lUpperSize - 1])))
			{
				// Remove the middle point of the three last
				lUpper[lUpperSize - 2] = lUpper[lUpperSize - 1];
				lUpperSize--;
			}
		}
		
		Vertex[] lLower = new Vertex[n];
		
		lLower[0] = xSorted.get(n - 1);
		lLower[1] = xSorted.get(n - 2);
		
		int lLowerSize = 2;
		
		for (int i = n - 3; i >= 0; i--)
		{
			lLower[lLowerSize] = xSorted.get(i);
			lLowerSize++;
			
			while (lLowerSize > 2 && !rightTurn(map.get(lLower[lLowerSize - 3]), map.get(lLower[lLowerSize - 2]), map.get(lLower[lLowerSize - 1])))
			{
				// Remove the middle point of the three last
				lLower[lLowerSize - 2] = lLower[lLowerSize - 1];
				lLowerSize--;
			}
		}
		
		ArrayList<Vertex> result = new ArrayList<Vertex>();
		
		for (int i = 0; i < lUpperSize; i++)
		{
			result.add(lUpper[i]);
		}
		
		for (int i = 1; i < lLowerSize - 1; i++)
		{
			result.add(lLower[i]);
		}
		
		for (int i=0; i<result.size(); i++){
			vertices.remove(result.get(i));
			vertices.add(0, result.get(i));
		}
		
		return result.size();
	}
	
	private boolean rightTurn(Point2D a, Point2D b, Point2D c)
	{
		return (b.getX() - a.getX())*(c.getY() - a.getY()) - (b.getY() - a.getY())*(c.getX() - a.getX()) > 0;
	}

	private class XCompare implements Comparator<Vertex>
	{
		HashMap<Vertex, Point2D> base;
		
		public XCompare(HashMap<Vertex, Point2D> base) {
			this.base = base;
		}
		
		public int compare(Vertex o1, Vertex o2) 
		{
			if (base.get(o1).getX() - base.get(o2).getX() > 0)
				return 1;
			else if (base.get(o1).getX() - base.get(o2).getX() == 0)
				return 0;
			else
				return -1;
		}
	}

}

