//package junk;

import java.util.*;
import java.awt.geom.Point2D;

public class GraphGraphics { 
    
  public static void drawWords(int m, String [] s, double [] x, double [] y){
	  StdDraw.setPenColor(StdDraw.BLACK);
      for (int i=0; i<m; i++){
        StdDraw.text(x[i], y[i], s[i]);
      }
  }   
    
  public static void drawWobblyLine(int wobble, double a, double b, double c, double d) {
    if (wobble == 0) {
        StdDraw.line(a,b,c,d);
    }
    else {
      double distance = Math.sqrt((a-c)*(a-c) + (b-d)*(b-d));
      double randMidPtx = (a + c)/2.0 + distance*(0.025 - 0.05*Math.random());
      double randMidPty = (b + d)/2.0 + distance*(0.025 - 0.05*Math.random());
      drawWobblyLine(wobble-1, a, b, randMidPtx, randMidPty);
      drawWobblyLine(wobble-1, randMidPtx, randMidPty, c, d);
    }  
  }
    
  public static void wobbleAround(int wobble, double a, double b) {
    double x0 = a;
    double y0 = b;
    double x1 = a;
    double y1 = b;
    for(int i=0; i<wobble; i++){
      double dx = 0.025 - 0.05*Math.random();
      double dy = 0.025 - 0.05*Math.random();
      x1 = a + dx;
      y1 = b + dy;
      StdDraw.line(x0,y0,x1,y1);
      x0 = x1;
      y0 = y1;
    }
    StdDraw.line(x1,y1,a,b);
  }
    
  public static void drawNodes(int m, double [] x, double [] y){  
      StdDraw.setPenColor(StdDraw.RED);
      for (int i=0; i<m; i++){
        StdDraw.filledCircle(x[i], y[i], 0.02);
      }
  }
  
  public static void drawNodes(SimpleGraph graph, HashMap<Vertex, Point2D> mapping){  
      StdDraw.setPenColor(StdDraw.RED);
      for (int i=0;i<graph.n();i++){
        StdDraw.filledCircle(mapping.get(graph.getVertices().get(i)).getX(), mapping.get(graph.getVertices().get(i)).getY(), 0.016);
      }
  }
  
  public static void drawNodeNames(int m, double [] x, double [] y, String [] s){  
      for (int i=0; i<m; i++){
        StdDraw.text(x[i], y[i], s[i]);
      }
  }
  
  public static void drawNodeNumbers(int m, double [] x, double [] y){  
      StdDraw.setPenColor(StdDraw.BLACK);
      for (int i=0; i<m; i++){
        StdDraw.text(x[i], y[i], String.valueOf(i));
      }
  }
  
  public static void drawNodeLabels(SimpleGraph graph, HashMap<Vertex, Point2D> mapping){  
      StdDraw.setPenColor(StdDraw.BLACK);
      for (int i=0; i<graph.n(); i++){
        StdDraw.text(mapping.get(graph.getVertices().get(i)).getX(), mapping.get(graph.getVertices().get(i)).getY(), graph.getVertices().get(i).getLabel());
      }
  }
  
  public static void drawEdges(int m, double [] x, double [] y, int [][] weight){  
    StdDraw.setPenColor(StdDraw.BLUE);
    for (int i=0; i<m; i++) for(int j=i+1; j<m; j++) {
      if ((j!=i)&&(weight[i][j]>0)) {
        StdDraw.line(x[i], y[i], x[j], y[j]);
      }
    }
  }
  
  public static void drawEdges(SimpleGraph graph, HashMap<Vertex, Point2D> mapping){  
	StdDraw.setPenColor(StdDraw.BLUE);
	int[][] g= graph.adjacencyMatrix();
	for (int i=0;i<graph.n();i++){
		for(int j=i+1; j<graph.n(); j++){
			if (g[i][j]>0) {
		        StdDraw.line(mapping.get(graph.getVertices().get(i)).getX(), mapping.get(graph.getVertices().get(i)).getY(),
		        		mapping.get(graph.getVertices().get(j)).getX(), mapping.get(graph.getVertices().get(j)).getY());
	        }
		}
    }    
  }
  
  public static void drawGraph(int n, double [] x, double [] y, int [][] adjacencyMatrix){  
	  drawEdges(n, x, y, adjacencyMatrix);
	  drawNodes(n, x, y);
	  drawNodeNumbers(n, x, y);
  }
  
  public static void drawGraph(SimpleGraph graph, HashMap<Vertex, Point2D> mapping){
	  drawEdges(graph, mapping);
	  drawNodes(graph, mapping);
	  drawNodeLabels(graph, mapping);
  }
 
  public static void scaleTo01Window(int m, double [] x, double [] y) {
    double maxx = 0.0;
    double minx = 1.0;
    double maxy = 0.0;
    double miny = 1.0;
    for (int i=0; i<m; i++) {
      if (x[i] > maxx) maxx = x[i];
      if (x[i] < minx) minx = x[i];
      if (y[i] > maxy) maxy = y[i];
      if (y[i] < miny) miny = y[i];
    }
    //System.out.printf("max min %f %f %f %f\n", maxx, minx, maxy, miny);
    for (int i=0; i<m; i++) {
      x[i] = (x[i] - minx)/(maxx - minx);
      y[i] = (y[i] - miny)/(maxy - miny);
    }
  }
   
//  public static void main (String [] args) {
//    int m = Integer.parseInt(args[1]);
//    int [][] weight = new int [m][m];
//    double [] x = new double[m];
//    double [] y = new double[m];
//    //springLayout.initLayout(m, x, y);
//    scaleTo01Window(m, x, y);
//    drawNodes(m, x, y);
//    drawEdges(m, x, y, weight);
//  }
    
}



