//package junk;

import java.util.HashMap;
import java.awt.geom.Point2D;

public class GraphTools {

	/********************************************
	 * 											*
	 *            Static variables				*
	 * 											*
	 ********************************************/
	
	static int [][] g;  // used for storing adjacency matrix of the graph
	
	static int n = 0;  // used for storing size of the graph
	static int numFixed = 0;  // number of fixed nodes in Tutte's algorithm
	
	static HashMap<Vertex, Point2D> tutteMap;  // maps each vertex to a geometric point 
	
	/********************************************
	 * 											*
	 *       Tutte's Drawing Algorithm			*
	 * 											*
	 ********************************************/
	
	// This method receives a graph and the number of fixed vertices as input and maps the vertices to geometric points on plane
	public static void tutte(SimpleGraph graph, int numFixed){
		g = graph.adjacencyMatrix();
		tutteMap = new HashMap<Vertex, Point2D>();
		
		// Initialising hash map
		for (int i=0; i<n; i++){ 
			tutteMap.put(graph.getVertices().get(i), new Point2D.Double());
		}

		for (int i=numFixed; i<n; i++){ 
			double theta = 2.0*Math.PI*Math.random();
			tutteMap.get(graph.getVertices().get(i)).setLocation(0.5 + 0.5*Math.cos(theta*i), 0.5 + 0.5*Math.sin(theta*i));
		}
		double theta = 2.0*Math.PI/ ((double) numFixed);
		for (int i=0; i<numFixed; i++){
			tutteMap.get(graph.getVertices().get(i)).setLocation(0.5 + 0.5*Math.cos(theta*i), 0.5 + 0.5*Math.sin(theta*i));
		}
		
		int k = 0;
		int staticCountLimit = 200*n;
		int staticCount = 0;
		double epsilon = 0.0001;
		while ((k<10000000)&&(staticCount < staticCountLimit)){
			for (int i=numFixed; i<n; i++) {
				
				double sumx = 0.0;
				double sumy = 0.0;
				double d = 0.0;
				for (int j=0; j<n; j++) {
					if (g[i][j] > 0) {
						sumx = sumx + g[i][j]*tutteMap.get(graph.getVertices().get(j)).getX();
						sumy = sumy + g[i][j]*tutteMap.get(graph.getVertices().get(j)).getY();
						d = d + g[i][j];
					}
				}
				double newX = sumx/d;
				double newY = sumy/d;
				if ((tutteMap.get(graph.getVertices().get(i)).getX()-newX)*(tutteMap.get(graph.getVertices().get(i)).getX()-newX) + (tutteMap.get(graph.getVertices().get(i)).getY()-newY)*(tutteMap.get(graph.getVertices().get(i)).getY()-newY) < epsilon) {
					staticCount++;
				}
				else {
					staticCount = 0;
				}

				tutteMap.get(graph.getVertices().get(i)).setLocation(newX, newY);
				k++;
			}
		}
	}

	// This method uses Tutte's algorithm and draws the graph which is given to it as input
	public static void drawTutteWeightedGraph (SimpleGraph graph, int numFixed) {
		n = graph.n();
		g = graph.adjacencyMatrix();

		graph.printGraph();
		tutte(graph,numFixed);

		GraphGraphics.drawGraph(graph, tutteMap);
	}
	
	/********************************************
	 * 											*
	 *               Main Method				*
	 * 											*
	 ********************************************/
	
	// Main method
	public static void main (String[] args) { 
		SimpleGraph graph = new SimpleGraph(args[0]);
		graph.printGraph();
		
		drawTutteWeightedGraph(graph, 3);
	}

}

